@@ -211,24 +211,43 @@ class AbsMilvus:
     #     pass
     #
     # @depracated
-    # def get_index_info(self, collection_name, field_name, index_name, timeout=30):
-    #     """
-    #     Show index information of a collection.
-    #
-    #     :type collection_name: str
-    #     :param collection_name: table name been queried
-    #
-    #     :returns:
-    #         IndexSchema:
-    #
-    #     :raises:
-    #         CollectionNotExistException(BaseException)
-    #         IllegalCollectionNameException(BaseException)
-    #         IllegalFieldNameException(BaseException)
-    #         IllegalIndexNameException(BaseException)
-    #
-    #     """
-    #     pass
+    def get_index_info(self, collection_name, field_name, timeout=30):
+        """
+        Show index information of a collection.
+
+        :type collection_name: str
+        :param collection_name: table name been queried
+
+        :returns:
+            IndexSchema:
+
+        :raises:
+            CollectionNotExistException(BaseException)
+            IllegalCollectionNameException(BaseException)
+            IllegalFieldNameException(BaseException)
+            IllegalIndexNameException(BaseException)
+
+        """
+        pass
+
+    def get_index_process(self, collection_name, field_name, timeout=30):
+        """
+        Show index information of a collection.
+
+        :type collection_name: str
+        :param collection_name: table name been queried
+
+        :returns:
+            IndexSchema:
+
+        :raises:
+            CollectionNotExistException(BaseException)
+            IllegalCollectionNameException(BaseException)
+            IllegalFieldNameException(BaseException)
+            IllegalIndexNameException(BaseException)
+
+        """
+        pass
 
     def drop_index(self, collection_name, field_name, index_name, timeout=30):
         """
