@@ -6,6 +6,7 @@ from ..client.exceptions import ParamError
 from .check import check_pass_param
 
 from .types import DataType
+from . import blob
 
 from ..grpc_gen import service_msg_pb2
 
@@ -73,6 +74,7 @@ class FieldSchema:
     def __pack(self, raw):
         self.name = raw.name
         self.description = raw.description
+        self.type = raw.data_type
         # self.type = DataType(int(raw.type))
 
         for type_param in raw.type_params:
@@ -87,7 +89,7 @@ class FieldSchema:
         _dict = dict()
         _dict["name"] = self.name
         _dict["description"] = self.description
-        # _dict["type"] = self.type
+        _dict["type"] = self.type
         _dict["params"] = self.params or dict()
         _dict["indexes"] = self.indexes
         return _dict
@@ -199,9 +201,10 @@ class Hit:
 
 
 class Hits(LoopBase):
-    def __init__(self, raw):
+    def __init__(self, raw, auto_id):
         super().__init__()
         self._raw = raw
+        self._auto_id = auto_id
         self._distances = self._raw.scores
         self._entities = []
         self._pack(self._raw)
@@ -219,6 +222,8 @@ class Hits(LoopBase):
         else:
             entity_row_data = None
         entity_score = self._raw.scores[item]
+        if not self._auto_id:
+            entity_id = blob.bytesToInt64(entity_row_data)
         return Hit(entity_id, entity_row_data, entity_score)
 
     @property
@@ -231,9 +236,10 @@ class Hits(LoopBase):
 
 
 class QueryResult(LoopBase):
-    def __init__(self, raw):
+    def __init__(self, raw, auto_id):
         super().__init__()
         self._raw = raw
+        self._auto_id = auto_id
         self._pack(raw.hits)
 
     def __len__(self):
@@ -252,7 +258,7 @@ class QueryResult(LoopBase):
         self._topk = len(self._hits[0].IDs) if len(self._hits) > 0 else 0
 
     def get__item(self, item):
-        return Hits(self._hits[item])
+        return Hits(self._hits[item], self._auto_id)
 
 
 def _abstract():
