@@ -356,6 +356,23 @@ class GrpcHandler(AbsMilvus):
             raise BaseException(response.status.error_code, response.status.reason)
         collection_schema = CollectionSchema(raw=response).dict()
 
+        auto_id = collection_schema["auto_id"]
+        fields_name = list()
+        for i in range(len(entities)):
+            if "name" in entities[i]:
+                fields_name.append(entities[i]["name"])
+        if not auto_id and ids is None and "_id" not in fields_name:
+            raise ParamError("You should specify the ids of entities!")
+
+        if not auto_id and ids is not None and "_id" in fields_name:
+            raise ParamError("You should specify the ids of entities!")
+
+        fields_info = collection_schema["fields"]
+
+        if (auto_id and len(entities) != len(fields_info)) \
+                or (not auto_id and ids is not None and len(entities) == len(fields_info)):
+            raise ParamError("The length of entities must be equal to the number of fields!")
+
         if ids is None:
             ids = []
             for entity in entities:
@@ -364,11 +381,6 @@ class GrpcHandler(AbsMilvus):
             if not ids:
                 ids = None
 
-        auto_id = collection_schema["auto_id"]
-        if not auto_id and ids is None:
-            raise ParamError("You should specify the ids of entities!")
-        fields_info = collection_schema["fields"]
-
         body = insert_param if insert_param \
             else Prepare.bulk_insert_param(collection_name, entities, partition_tag, ids, params, fields_info, auto_id=auto_id)
 
@@ -401,6 +413,26 @@ class GrpcHandler(AbsMilvus):
             raise BaseException(response.status.error_code, response.status.reason)
         collection_schema = CollectionSchema(raw=response).dict()
 
+        auto_id = collection_schema["auto_id"]
+        fields_name = list()
+        entity = entities[0]
+        for key, value in entity.items():
+            if key in fields_name:
+                raise ParamError("duplicated field name in entity")
+            fields_name.append(key)
+
+        if not auto_id and ids is None and "_id" not in fields_name:
+            raise ParamError("You should specify the ids of entities!")
+
+        if not auto_id and ids is not None and "_id" in fields_name:
+            raise ParamError("You should specify the ids of entities!")
+
+        fields_info = collection_schema["fields"]
+
+        if (auto_id and len(entities[0]) != len(fields_info)) \
+                or (not auto_id and ids is not None and len(entities[0]) == len(fields_info)):
+            raise ParamError("The length of entities must be equal to the number of fields!")
+
         if ids is None:
             ids = []
             for entity in entities:
@@ -409,11 +441,6 @@ class GrpcHandler(AbsMilvus):
             if not ids:
                 ids = None
 
-        auto_id = collection_schema["auto_id"]
-        if not auto_id and ids is None:
-            raise ParamError("You should specify the ids of entities!")
-        fields_info = collection_schema["fields"]
-
         body = insert_param if insert_param \
             else Prepare.insert_param(collection_name, entities, partition_tag, ids, params, fields_info, auto_id=auto_id)
 
@@ -454,7 +481,58 @@ class GrpcHandler(AbsMilvus):
         if response.status.error_code != 0:
             raise BaseException(response.status.error_code, response.status.reason)
 
+        collection_schema = self.get_collection_info(collection_name=collection_name)
+        auto_id = collection_schema["auto_id"]
+
         # TODO: handler response
         # resutls = self._search_hook.handle_response(response)
 
-        return QueryResult(response)
+        return QueryResult(response, auto_id)
+
+    @error_handler()
+    def create_index(self, collection_name, field_name, params, timeout=None, **kwargs):
+        index_param = Prepare.index_param(collection_name, field_name, params)
+        future = self._stub.CreateIndex.future(index_param, wait_for_ready=True, timeout=timeout)
+        if kwargs.get('_async', False):
+            cb = kwargs.get("_callback", None)
+            return CreateIndexFuture(future, cb)
+        status = future.result()
+
+        if status.error_code != 0:
+            raise BaseException(status.error_code, status.reason)
+
+        return Status(status.error_code, status.reason)
+
+    @error_handler(None)
+    def get_index_info(self, collection_name, field_name, timeout=None, **kwargs):
+        request = Prepare.describe_index_request(collection_name, field_name)
+
+        rf = self._stub.DescribeIndex.future(request, wait_for_ready=True, timeout=timeout)
+        response = rf.result()
+        status = response.status
+        if status.error_code == 0:
+            extra_params = response.extra_params
+            info_dict = dict()
+            for kv in extra_params:
+                info_dict[kv.key] = kv.value
+            return info_dict
+        raise BaseException(status.error_code, status.reason)
+
+    @error_handler(False)
+    def get_index_progress(self, collection_name, field_name, timeout=None, **kwargs):
+        request = Prepare.describe_index_progress_request(collection_name, field_name)
+
+        rf = self._stub.DescribeIndexProgress.future(request, wait_for_ready=True, timeout=timeout)
+        response = rf.result()
+        status = response.status
+        if status.error_code == 0:
+            return response.value
+        raise BaseException(status.error_code, status.reason)
+
+    @error_handler()
+    def wait_index_building_success(self, collection_name, field_name):
+        while True:
+            if self.get_index_progress(collection_name=collection_name, field_name=field_name):
+                break
+
+        return
