@@ -70,6 +70,15 @@ class Prepare:
         auto_id = fields.get('auto_id', True)
         schema.autoID = auto_id
 
+        if not auto_id:
+            field_schema = schema_types.FieldSchema()
+
+            field_schema.name = "_id"
+            field_schema.is_primary_key = True
+            field_schema.description = "this is _id field"
+            field_schema.data_type = DataType.INT64
+            schema.fields.append(field_schema)
+
         for fk, fv in fields.items():
             if fk != 'fields':
                 # TODO: add extra_params
@@ -144,12 +153,16 @@ class Prepare:
         row_batch.partition_tag = partition_tag or default_partition_tag
 
         row_data = list()
+        fields_name = list()
         fields_type = list()
         fields_len = len(entities)
         for i in range(fields_len):
-            fields_type.append(entities[i]["type"])
+            fields_name.append(entities[i]["name"])
+
         row_num = len(entities[0]["values"]) if fields_len > 0 else 0
 
+        auto_id = kwargs.get("auto_id", True)
+
         if fields_info is None:
             # this case, we assume the order of entities is same to schema
             for i in range(row_num):
@@ -160,30 +173,42 @@ class Prepare:
         else:
             # field name & type must match fields info
             location = dict()
-            if fields_len != len(fields_info):
-                raise ParamError("The length of entities must be equal to the number of fields!")
+
             for i, field in enumerate(fields_info):
                 match_flag = False
                 for j in range(fields_len):
                     # if field["name"] == entities[j]["name"] and field["type"] == entities[j]["type"]:
-                    if field["name"] == entities[j]["name"]:
-                        location[field["name"]] = j
-                        fields_type[i] = entities[j]["type"]
-                        match_flag = True
-                        break
+                    if not auto_id and "_id" not in fields_name:
+                        if field["name"] == "_id":
+                            match_flag = True
+                            break
+                        if field["name"] == entities[j]["name"]:
+                            location[field["name"]] = j
+                            fields_type.append(entities[j]["type"])
+                            match_flag = True
+                            break
+
+                    else:
+                        if field["name"] == entities[j]["name"]:
+                            location[field["name"]] = j
+                            fields_type.append(entities[j]["type"])
+                            match_flag = True
+                            break
                 if not match_flag:
                     raise ParamError("Field {} don't match in entities".format(field["name"]))
             for i in range(row_num):
                 row = []
-                for j in range(fields_len):
+                for j in range(len(fields_info)):
                     field_name = fields_info[j]["name"]
-                    loc = location[field_name]
-                    row.append(entities[loc]["values"][i])
+                    if not auto_id and "_id" not in fields_name and field_name == "_id":
+                        continue
+                    else:
+                        loc = location[field_name]
+                        row.append(entities[loc]["values"][i])
                 row_data.append(row)
 
         # fill row_data in bytes format
-        auto_id = kwargs.get("auto_id", True)
-        if not auto_id:
+        if not auto_id and "_id" not in fields_name:
             id_type = DataType.INT64  # int64_t is supported by default
             fields_type.insert(0, id_type)
             for i in range(row_num):
@@ -247,14 +272,19 @@ class Prepare:
                 names.insert(0, "_id")
             return names
 
-        field_names = list()
-        if fields_info is None or True:
+        # this case, we assume the order of entities is same to schema
+        fields_name = list()
+        entity = entities[0]
+        for key, value in entity.items():
+            if key in fields_name:
+                raise ParamError("duplicated field name in entity")
+            fields_name.append(key)
+
+        auto_id = kwargs.get("auto_id", True)
+
+        if fields_info is None:
             # this case, we assume the order of entities is same to schema
-            entity = entities[0]
             for key, value in entity.items():
-                if key in field_names:
-                    raise ParamError("duplicated field name in entity")
-                field_names.append(key)
                 if isinstance(value, bool):
                     fields_type.append(DataType.BOOL)
                 elif isinstance(value, int):
@@ -275,24 +305,36 @@ class Prepare:
                     raise ParamError("unsupported data type")
             # row_data = [[value for _, value in entity.items()]
             #             for entity in entities]
-            row_data = list()
-            schema_field_names = get_fields_by_schema() or field_names
+
+            schema_field_names = get_fields_by_schema() or fields_name
             for entity in entities:
                 row = list()
                 for key, value in entity.items():
-                    if key not in field_names:
+                    if key not in fields_name:
                         raise ParamError("entities has different field name")
                     if key not in schema_field_names:
                         raise ParamError("field name must be in schema")
                     row.append(value)
                 row_data.append(row)
         else:
-            # fields_info now doesn't contain type info
-            pass
+            # field name & type must match fields info
+            for j in range(len(fields_info)):
+                field_name = fields_info[j]["name"]
+                if not auto_id and "_id" not in fields_name and field_name == "_id":
+                    continue
+                elif field_name not in fields_name:
+                    raise ParamError("Field {} don't match in entities".format(field_name))
+                fields_type.append(fields_info[j]["type"])
+
+            for i in range(row_num):
+                row = list()
+                for j in range(len(fields_info)):
+                    field_name = fields_info[j]["name"]
+                    row.append(entities[i][field_name])
+                row_data.append(row)
 
         # fill row_data in bytes format
-        auto_id = kwargs.get("auto_id", True)
-        if not auto_id:
+        if not auto_id and "_id" not in fields_name:
             id_type = DataType.INT64  # int64_t is supported by default
             fields_type.insert(0, id_type)
             for i in range(row_num):
@@ -394,3 +436,29 @@ class Prepare:
         query.placeholder_group = plg_str
 
         return query
+
+    @classmethod
+    def index_param(cls, collection_name, field_name, params):
+        index_params = service_msg_types.IndexParam(collection_name=collection_name, field_name=field_name)
+        # index_params.collection_name = collection_name
+        # index_params.field_name = field_name
+
+        if isinstance(params, dict):
+            for tk, tv in params.items():
+                if tk == "dim":
+                    if not tv or not isinstance(tv, int):
+                        raise ParamError("dim must be of int!")
+                kv_pair = common_types.KeyValuePair(key=str(tk), value=str(tv))
+                index_params.extra_params.append(kv_pair)
+
+        return index_params
+
+    @classmethod
+    def describe_index_request(cls, collection_name, field_name):
+
+        return service_msg_types.DescribeIndexRequest(collection_name, field_name)
+
+    @classmethod
+    def describe_index_progress_request(cls, collection_name, field_name):
+
+        return service_msg_types.DescribeIndexProgressRequest(collection_name, field_name)
